@@ -1,12 +1,10 @@
 """Test command group - test code examples with upstream credentials."""
 
 import fnmatch
-import json
 import os
 import shutil
 import subprocess
 import tempfile
-import tomllib
 from pathlib import Path
 from typing import Any
 
@@ -154,26 +152,27 @@ def load_related_data(listing_file: Path) -> dict[str, Any]:
     return result
 
 
-def load_provider_credentials(provider_file: Path) -> dict[str, str] | None:
-    """Load API key and endpoint from provider file.
+def load_provider_credentials(listing_file: Path) -> dict[str, str] | None:
+    """Load API key and endpoint from service offering file.
 
     Args:
-        provider_file: Path to provider.toml or provider.json
+        listing_file: Path to the listing file (used to locate the service offering)
 
     Returns:
         Dictionary with api_key and api_endpoint, or None if not found
     """
     try:
-        if provider_file.suffix == ".toml":
-            with open(provider_file, "rb") as f:
-                provider_data = tomllib.load(f)
-        else:
-            with open(provider_file) as f:
-                provider_data = json.load(f)
+        # Load related data including the offering
+        related_data = load_related_data(listing_file)
+        offering = related_data.get("offering", {})
 
-        access_info = provider_data.get("provider_access_info", {})
-        api_key = access_info.get("api_key") or access_info.get("FIREWORKS_API_KEY")
-        api_endpoint = access_info.get("api_endpoint") or access_info.get("FIREWORKS_API_BASE_URL")
+        if not offering:
+            return None
+
+        # Extract credentials from upstream_access_interface
+        upstream_access = offering.get("upstream_access_interface", {})
+        api_key = upstream_access.get("api_key")
+        api_endpoint = upstream_access.get("api_endpoint")
 
         if api_key and api_endpoint:
             return {
@@ -181,7 +180,7 @@ def load_provider_credentials(provider_file: Path) -> dict[str, str] | None:
                 "api_endpoint": str(api_endpoint),
             }
     except Exception as e:
-        console.print(f"[yellow]Warning: Failed to load provider credentials: {e}[/yellow]")
+        console.print(f"[yellow]Warning: Failed to load service credentials: {e}[/yellow]")
 
     return None
 
@@ -473,9 +472,14 @@ def list_code_examples(
         code_examples = extract_code_examples_from_listing(listing_data, listing_file)
 
         for example in code_examples:
-            # Get file extension
+            # Get file extension (strip .j2 if present to show actual type)
             file_path = example.get("file_path", "")
-            file_ext = Path(file_path).suffix or "unknown"
+            path = Path(file_path)
+            # If it's a .j2 template, get the extension before .j2
+            if path.suffix == ".j2":
+                file_ext = Path(path.stem).suffix or "unknown"
+            else:
+                file_ext = path.suffix or "unknown"
             all_code_examples.append((example, prov_name, file_ext))
 
     if not all_code_examples:
@@ -535,6 +539,12 @@ def run(
         "-s",
         help="Comma-separated list of service patterns (supports wildcards, e.g., 'llama*,gpt-4*')",
     ),
+    test_file: str | None = typer.Option(
+        None,
+        "--test-file",
+        "-t",
+        help="Only run a specific test file by filename (e.g., 'code-example.py.j2')",
+    ),
     verbose: bool = typer.Option(
         False,
         "--verbose",
@@ -564,6 +574,9 @@ def run(
         # Test single service
         unitysvc_services test run --services "llama-3-1-405b-instruct"
 
+        # Test specific file
+        unitysvc_services test run --test-file "code-example.py.j2"
+
         # Combine filters
         unitysvc_services test run --provider fireworks --services "llama*"
 
@@ -590,30 +603,12 @@ def run(
         service_patterns = [s.strip() for s in services.split(",") if s.strip()]
         console.print(f"[blue]Service filter patterns:[/blue] {', '.join(service_patterns)}\n")
 
+    # Display test file filter if provided
+    if test_file:
+        console.print(f"[blue]Test file filter:[/blue] {test_file}\n")
+
     console.print(f"[blue]Scanning for listing files in:[/blue] {data_dir}\n")
 
-    # Find all provider files first to get credentials
-    provider_results = find_files_by_schema(data_dir, "provider_v1")
-    provider_credentials: dict[str, dict[str, str]] = {}
-
-    for provider_file, _format, provider_data in provider_results:
-        prov_name = provider_data.get("name", "unknown")
-
-        # Skip if provider filter is set and doesn't match
-        if provider_name and prov_name != provider_name:
-            continue
-
-        credentials = load_provider_credentials(provider_file)
-        if credentials:
-            provider_credentials[prov_name] = credentials
-            console.print(f"[green]✓[/green] Loaded credentials for provider: {prov_name}")
-
-    if not provider_credentials:
-        console.print("[yellow]No provider credentials found.[/yellow]")
-        raise typer.Exit(code=0)
-
-    console.print()
-
     # Find all listing files
     listing_results = find_files_by_schema(data_dir, "listing_v1")
 
@@ -624,7 +619,7 @@ def run(
     console.print(f"[cyan]Found {len(listing_results)} listing file(s)[/cyan]\n")
 
     # Extract code examples from all listings
-    all_code_examples: list[tuple[dict[str, Any], str]] = []
+    all_code_examples: list[tuple[dict[str, Any], str, dict[str, str]]] = []
 
     for listing_file, _format, listing_data in listing_results:
         # Determine provider for this listing
@@ -643,8 +638,10 @@ def run(
         if provider_name and prov_name != provider_name:
             continue
 
-        # Skip if we don't have credentials for this provider
-        if prov_name not in provider_credentials:
+        # Load credentials from service offering for this listing
+        credentials = load_provider_credentials(listing_file)
+        if not credentials:
+            console.print(f"[yellow]⚠ No credentials found for listing: {listing_file}[/yellow]")
             continue
 
         # Filter by service directory name if patterns are provided
@@ -661,7 +658,14 @@ def run(
         code_examples = extract_code_examples_from_listing(listing_data, listing_file)
 
         for example in code_examples:
-            all_code_examples.append((example, prov_name))
+            # Filter by test file name if provided
+            if test_file:
+                file_path = example.get("file_path", "")
+                # Check if the file path ends with the test file name
+                if not file_path.endswith(test_file):
+                    continue
+
+            all_code_examples.append((example, prov_name, credentials))
 
     if not all_code_examples:
         console.print("[yellow]No code examples found in listings.[/yellow]")
@@ -672,13 +676,12 @@ def run(
     # Execute each code example
     results = []
 
-    for example, prov_name in all_code_examples:
+    for example, prov_name, credentials in all_code_examples:
         service_name = example["service_name"]
         title = example["title"]
 
         console.print(f"[bold]Testing:[/bold] {service_name} - {title}")
 
-        credentials = provider_credentials[prov_name]
         result = execute_code_example(example, credentials)
 
         results.append(
@@ -695,26 +698,38 @@ def run(
             if verbose and result["stdout"]:
                 console.print(f"  [dim]stdout:[/dim] {result['stdout'][:200]}")
 
-            # Save successful test output to .out file
-            if result.get("stdout") and result.get("listing_file") and result.get("actual_filename"):
+            # Save successful test output to .out and .err files
+            if result.get("listing_file") and result.get("actual_filename"):
                 listing_file = Path(result["listing_file"])
                 actual_filename = result["actual_filename"]
-
-                # Create filename: {listing_stem}_{actual_filename}.out
-                # e.g., "svclisting_test.py.out" for svclisting.json and test.py
                 listing_stem = listing_file.stem
-                output_filename = f"{listing_stem}_{actual_filename}.out"
+
+                # Create filename pattern: {service_name}_{listing_stem}_{actual_filename}.out/.err
+                # e.g., "llama-3-1-405b-instruct_svclisting_test.py.out"
+                base_filename = f"{service_name}_{listing_stem}_{actual_filename}"
+                out_filename = f"{base_filename}.out"
+                err_filename = f"{base_filename}.err"
 
                 # Save to listing directory
-                output_path = listing_file.parent / output_filename
+                out_path = listing_file.parent / out_filename
+                err_path = listing_file.parent / err_filename
 
                 # Write stdout to .out file
                 try:
-                    with open(output_path, "w", encoding="utf-8") as f:
-                        f.write(result["stdout"])
-                    console.print(f"  [dim]→ Output saved to:[/dim] {output_path}")
+                    with open(out_path, "w", encoding="utf-8") as f:
+                        f.write(result["stdout"] or "")
+                    console.print(f"  [dim]→ Output saved to:[/dim] {out_path}")
                 except Exception as e:
                     console.print(f"  [yellow]⚠ Failed to save output: {e}[/yellow]")
+
+                # Write stderr to .err file
+                try:
+                    with open(err_path, "w", encoding="utf-8") as f:
+                        f.write(result["stderr"] or "")
+                    if result["stderr"]:
+                        console.print(f"  [dim]→ Error output saved to:[/dim] {err_path}")
+                except Exception as e:
+                    console.print(f"  [yellow]⚠ Failed to save error output: {e}[/yellow]")
         else:
             console.print(f"  [red]✗ Failed[/red] - {result['error']}")
             if verbose:
@@ -723,16 +738,36 @@ def run(
                 if result["stderr"]:
                     console.print(f"  [dim]stderr:[/dim] {result['stderr'][:200]}")
 
-            # Write failed test content to current directory
-            if result.get("rendered_content") and result.get("listing_file") and result.get("actual_filename"):
+            # Write failed test outputs and script to current directory
+            if result.get("listing_file") and result.get("actual_filename"):
                 listing_file = Path(result["listing_file"])
                 actual_filename = result["actual_filename"]
                 listing_stem = listing_file.stem
 
-                # Create filename: failed_{listing_stem}_{actual_filename}
-                failed_filename = f"failed_{listing_stem}_{actual_filename}"
+                # Create filename: failed_{service_name}_{listing_stem}_{actual_filename}
+                # This will be the base name for .out, .err, and the script file
+                failed_filename = f"failed_{service_name}_{listing_stem}_{actual_filename}"
 
-                # Prepare content with environment variables as header comments
+                # Write stdout to .out file in current directory
+                out_filename = f"{failed_filename}.out"
+                try:
+                    with open(out_filename, "w", encoding="utf-8") as f:
+                        f.write(result["stdout"] or "")
+                    console.print(f"  [yellow]→ Output saved to:[/yellow] {out_filename}")
+                except Exception as e:
+                    console.print(f"  [yellow]⚠ Failed to save output: {e}[/yellow]")
+
+                # Write stderr to .err file in current directory
+                err_filename = f"{failed_filename}.err"
+                try:
+                    with open(err_filename, "w", encoding="utf-8") as f:
+                        f.write(result["stderr"] or "")
+                    console.print(f"  [yellow]→ Error output saved to:[/yellow] {err_filename}")
+                except Exception as e:
+                    console.print(f"  [yellow]⚠ Failed to save error output: {e}[/yellow]")
+
+                # Write failed test script content to current directory (for debugging)
+                # rendered_content is always set if we got here (set during template rendering)
                 content_with_env = result["rendered_content"]
 
                 # Add environment variables as comments at the top
@@ -749,14 +784,13 @@ def run(
 
                 content_with_env = env_header + content_with_env
 
-                # Write to current directory
                 try:
                     with open(failed_filename, "w", encoding="utf-8") as f:
                         f.write(content_with_env)
-                    console.print(f"  [yellow]→ Test content saved to:[/yellow] {failed_filename}")
+                    console.print(f"  [yellow]→ Test script saved to:[/yellow] {failed_filename}")
                     console.print("  [dim]  (includes environment variables for reproduction)[/dim]")
                 except Exception as e:
-                    console.print(f"  [yellow]⚠ Failed to save test content: {e}[/yellow]")
+                    console.print(f"  [yellow]⚠ Failed to save test script: {e}[/yellow]")
 
         console.print()
 
